﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace InitializerModel
{
    public static class EvseVersion
    {
        public static int GetVersionIndex(string version)
        {
            if (version.StartsWith("DD"))
            {
                var dispensorIndexChar = version.Last();
                if (int.TryParse(dispensorIndexChar.ToString(), out var result))
                {
                    return result + 1;
                }
            }
            return 0;
        }

        public static string GetDDApiNameFromApiName(string apiName, int index)
        {
            if (index <= 0)
                return apiName;
            return $"DD{apiName}{index - 1}";
        }

        public static string GetDDApiNameFromWebName(string webName, int index)
        {
            var apiName = PredefinedVersionPair.GetApiName(webName);
            return GetDDApiNameFromApiName(apiName, index);
        }

        public static string GetApiNameFromDDApiName(string apiName)
        {
            var index = GetVersionIndex(apiName);
            if (index == 0)
            {
                return apiName;
            }
            return apiName.Substring(2, apiName.Length -3);
        }
    }

    public static class PredefinedVersionPair
    {
        public static IEnumerable<string> GetWebNames()
        {
            return PreDefinedVersions.Select(x => x.WebName);
        }

        public static IEnumerable<string> GetApiNames()
        {
            return PreDefinedVersions.Select(x => x.ApiName);
        }

        public static string GetWebName(string apiName)
        {
            return PreDefinedVersions.FirstOrDefault(x => x.ApiName == apiName)?.WebName;
        }

        public static string GetApiName(string webName)
        {
            return PreDefinedVersions.FirstOrDefault(x => x.WebName == webName)?.ApiName;
        }

        private static List<VersionDefinePair> PreDefinedVersions = new List<VersionDefinePair>(){
                new VersionDefinePair("Csu Boot Load Fw Rev","CsuBootLoadFwRev" ),
                new VersionDefinePair("Csu Kernel Fw Rev"   ,"CsuKernelFwRev" ),
                new VersionDefinePair("Csu Root Fs Fw Rev"  ,"CsuRootFsFwRev" ),
                new VersionDefinePair("Csu Prim Fw Rev"     ,"CsuPrimFwRev" ),
                new VersionDefinePair("Psu Prim Fw Rev(1)"  ,"FwPrimaryVersion0" ),
                new VersionDefinePair("Psu Sec Fw Rev(1)"   ,"FwSecondVersion0" ),
                new VersionDefinePair("Psu Prim Fw Rev(2)"  ,"FwPrimaryVersion1" ),
                new VersionDefinePair("Psu Sec Fw Rev(2)"   ,"FwSecondVersion1" ),
                new VersionDefinePair("Fan Module Fw Rev"   ,"FanModuleFwRev" ),
                new VersionDefinePair("Relay Module Fw Rev" ,"RelayModuleFwRev" ),
                new VersionDefinePair("Relay2 Module Fw Rev","Relay2ModuleFwRev" ),
                new VersionDefinePair("Led Module Fw Rev"   ,"LedModuleFwRev" ),
                new VersionDefinePair("Connector1 Fw Rev"   ,"Connector1FwRev" ),
                new VersionDefinePair("Connector2 Fw Rev"   ,"Connector2FwRev" ),
            };

    }

    public class VersionDefinePair
    {
        public string WebName { get; set; }
        public string ApiName { get; set; }

        public VersionDefinePair(string webName, string apiName)
        {
            WebName = webName;
            ApiName = apiName;
        }
    }
}
